#include "StdAfx.h"
#include "WindowMain.h"

using namespace Softphone_VC_NET;

void WindowMain::WindowMain_Load(System::Object^  sender, System::EventArgs^  e)
{
	Tssl_Ozeki->Text = String::Format(" Copyright {0}, Ozeki Informatics Ltd.", DateTime::Now.Year);

	InitMediaHandlers();
	InitializeSoftphone();
}

// It initializes a softphone object with a SIP BPX, and it is for requisiting a SIP account that is nedded for a SIP PBX service. It registers this SIP
// account to the SIP PBX through an IphoneLine object which represents the telephoneline. 
// If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
void WindowMain::InitializeSoftphone()
{
	try
	{
		m_softPhone = SoftPhoneFactory::CreateSoftPhone(SoftPhoneFactory::GetLocalIP(), 5000, 6000, 5060, nullptr);
		m_softPhone->IncomingCall += gcnew EventHandler<VoIPEventArgs<IPhoneCall^>^>(this, &WindowMain::softPhone_IncomingCall);

		auto sipAccount = gcnew SIPAccount(true, "oz875", "oz875", "oz875", "oz875", "192.168.112.100");
		m_phoneLine = m_softPhone->CreatePhoneLine(sipAccount, TransportType::Udp, SRTPMode::None);
		m_phoneLine->PhoneLineStateChanged += gcnew EventHandler<VoIPEventArgs<PhoneLineState>^>(this, &WindowMain::phoneLine_PhoneLineInformation);
		m_softPhone->RegisterPhoneLine(m_phoneLine, nullptr);
	}
	catch (Exception ^ex)
	{
		StringBuilder sb;

        sb.AppendLine("Some error happened.");
        sb.AppendLine();
        sb.AppendLine("Exception:");
        sb.AppendLine(ex->Message);
        sb.AppendLine();
        if(ex->InnerException != nullptr)
        {
            sb.AppendLine("Inner Exception:");
            sb.AppendLine(ex->InnerException->Message);
            sb.AppendLine();
        }

        sb.AppendLine("StackTrace:");
        sb.AppendLine(ex->StackTrace);

		
		MessageBox::Show(sb.ToString());	
	}
}

void WindowMain::InitMediaHandlers()
{
	m_microphone = Microphone::GetDefaultDevice();
	m_speaker = Speaker::GetDefaultDevice();

	m_connector = gcnew MediaConnector();
	m_mediaSender = gcnew PhoneCallAudioSender();
	m_mediaReceiver = gcnew PhoneCallAudioReceiver();
}

//Occurs when an incoming call request has received.
void WindowMain::softPhone_IncomingCall(Object ^sender, VoIPEventArgs<IPhoneCall^> ^e)
{
	SetIncomingCall(e->Item);
}

void WindowMain::SetIncomingCall(IPhoneCall ^phoneCall)
{
    if(this->InvokeRequired)
	{
		auto handler = gcnew SetIncomingCallDelegate(this, &WindowMain::SetIncomingCall);
		this->BeginInvoke(handler, phoneCall);
	}
	else
	{
		labelCallStateInfo->Text = "Incoming call";
		labelDialingNumber->Text = String::Format("from {0}", phoneCall->DialInfo);
		m_call = phoneCall;
		m_incomingCall = true;
	}

    
	WireUpCallEvents();

}

void WindowMain::WireUpCallEvents()
{
	m_call->CallStateChanged += gcnew EventHandler<VoIPEventArgs<CallState>^>(this, &WindowMain::call_CallStateChanged);
	m_call->DtmfReceived += gcnew EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal^>^>^>(this, &WindowMain::call_DtmfReceived);
	m_call->CallErrorOccured += gcnew EventHandler<VoIPEventArgs<CallError>^>(this, &WindowMain::call_ErrorOccurred);
}

void WindowMain::WireDownCallEvents()
{
	m_call->CallStateChanged -= gcnew EventHandler<VoIPEventArgs<CallState>^>(this, &WindowMain::call_CallStateChanged);
	m_call->DtmfReceived -= gcnew EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal^>^>^>(this, &WindowMain::call_DtmfReceived);
	m_call->CallErrorOccured -= gcnew EventHandler<VoIPEventArgs<CallError>^>(this, &WindowMain::call_ErrorOccurred);
}

//Displays DTMF signals
void WindowMain::call_DtmfReceived(Object ^sender, VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal^>^> ^e)
{
	auto text = String::Format("DTMF signal received: {0} ", e->Item->Item2->Signal);
	SetCallStateInfo(text);
}

//Occurs when the phone call state has changed.
void WindowMain::call_CallStateChanged(Object ^sender, VoIPEventArgs<CallState> ^e)
{
	if(this->InvokeRequired)
	{
		auto handler = gcnew SetTextDelegate(this, &WindowMain::SetCallStateInfo);
		BeginInvoke(handler, e->Item.ToString());
	}
	else
		SetCallStateInfo(e->Item.ToString());

	switch (e->Item)
    {
		case CallState::InCall:
            m_microphone->Start();
            m_connector->Connect(m_microphone, m_mediaSender);

            m_speaker->Start();
            m_connector->Connect(m_mediaReceiver, m_speaker);
                    
            m_mediaSender->AttachToCall(m_call);
            m_mediaReceiver->AttachToCall(m_call);

            break;
		case CallState::Completed:
            m_microphone->Stop();
            m_connector->Disconnect(m_microphone, m_mediaSender);

            m_speaker->Stop();
            m_connector->Disconnect(m_mediaReceiver, m_speaker);

            m_mediaSender->Detach();
            m_mediaReceiver->Detach();

            WireDownCallEvents();
            m_call = nullptr;
			SetDialingNumber(String::Empty);
		case CallState::Cancelled:
            WireDownCallEvents();
            m_call = nullptr;
			SetDialingNumber(String::Empty);
            break;
    }
}

// There are certain situations when the call cannot be created, for example the dialed number is not available 
// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
// This event handling is for displaying these events.
void WindowMain::call_ErrorOccurred(Object ^sender, VoIPEventArgs<CallError> ^e)
{
	if(this->InvokeRequired)
	{
		auto handler = gcnew SetTextDelegate(this, &WindowMain::SetCallStateInfo);
		BeginInvoke(handler, e->Item.ToString());
	}
	else
		SetCallStateInfo(e->Item.ToString());
}

//Occurs when phone line state has changed.
void WindowMain::phoneLine_PhoneLineInformation(Object ^sender, VoIPEventArgs<PhoneLineState> ^e)
{
	if(this->InvokeRequired)
	{
		auto handler = gcnew SetPhoneLineStateDelegate(this, &WindowMain::SetPhoneLineState);
		this->BeginInvoke(handler, e->Item);
	}
	else
	{
		SetPhoneLineState(e->Item);
	}
}

//It stops sending the given DTMF signal.
void WindowMain::btn_MouseUp(System::Object ^sender, System::Windows::Forms::MouseEventArgs ^e)
{
	if (m_call != nullptr && m_call->CallState == CallState::InCall)
	{
		auto btn = dynamic_cast<Button^>(sender);
		if (btn != nullptr)
		{
			int id;
			if (btn->Tag != nullptr && int::TryParse(btn->Tag->ToString(), id))
			{
				m_call->StopDTMFSignal(VoIPMediaType::Audio, static_cast<DtmfNamedEvents>(id));
            }
        }
    }
}

//It sends a DTMF signal according to the RFC 2833 standard.
void WindowMain::btn_MouseDown(System::Object ^sender, System::Windows::Forms::MouseEventArgs ^e)
{
	if (m_call != nullptr && m_call->CallState == CallState::InCall)
	{
		auto btn = dynamic_cast<Button^>(sender);
		if (btn != nullptr)
		{
			int id;
			if (btn->Tag != nullptr && int::TryParse(btn->Tag->ToString(), id))
			{
				m_call->StopDTMFSignal(VoIPMediaType::Audio, static_cast<DtmfNamedEvents>(id));
			}
		}
    }
}

//It starts a call with the dialed number or in case of an incoming call it accepts, picks up the call.
void WindowMain::buttonPickUp_Click(System::Object ^sender, System::EventArgs ^e)
{
	if (m_incomingCall)
	{
		m_incomingCall = false;
        m_call->Accept();
        return;
    }

    if (m_call != nullptr)
        return;

    if (String::IsNullOrEmpty(labelDialingNumber->Text))
        return;

    if (m_phoneLineInformation != PhoneLineState::RegistrationSucceeded && m_phoneLineInformation != PhoneLineState::NoRegNeeded)
    {
        MessageBox::Show("Phone line state is not valid!");
        return;
    }

    m_call = m_softPhone->CreateCallObject(m_phoneLine, labelDialingNumber->Text, nullptr);
    WireUpCallEvents();
    m_call->Start();
}

//In case a call is in progress, it breaks the call.
void WindowMain::buttonHangUp_Click(System::Object ^sender, System::EventArgs ^e)
{
	if (m_call != nullptr)
	{   
		if (m_incomingCall && m_call->CallState == CallState::Ringing)
			m_call->Reject();
        else
			m_call->HangUp();
        
		m_incomingCall = false;
        m_call = nullptr;
     }
     labelDialingNumber->Text = String::Empty;
}

//It makes the dialing number.
void WindowMain::btn_Click(System::Object ^sender, System::EventArgs ^e)
{
	auto btn = dynamic_cast<Button^>(sender);
	
	if (btn != nullptr)
		labelDialingNumber->Text += btn->Tag;
}

void WindowMain::SetCallStateInfo(String ^text)
{
	if(this->InvokeRequired)
	{
		auto handler = gcnew SetTextDelegate(this, &WindowMain::SetCallStateInfo); 
		BeginInvoke(handler, text);
	}
	else
	{
		labelCallStateInfo->Text = text;
	}

}

void WindowMain::SetDialingNumber(String ^text)
{
	if(this->InvokeRequired)
	{
		auto handler = gcnew SetTextDelegate(this, &WindowMain::SetDialingNumber);
		BeginInvoke(handler, text);
	}
	else
	{
		labelDialingNumber->Text = text;
	}

}

void WindowMain::SetPhoneLineState(PhoneLineState phoneLineState)
{
	m_phoneLineInformation = phoneLineState;

	labelIdentifier->Text = m_phoneLine->SIPAccount->RegisterName;
	if (phoneLineState == PhoneLineState::RegistrationSucceeded)
	{
		labelRegStatus->Text = "Online";
		labelCallStateInfo->Text = "Registration succeeded";
    }
	else
		labelCallStateInfo->Text = phoneLineState.ToString();
}

void WindowMain::exitToolStripMenuItem_Click(System::Object ^sender, System::EventArgs ^e)
{
	Application::Exit();
}

void WindowMain::Tsmi_HelpPage_Click(System::Object^  sender, System::EventArgs^  e)
{
	try
	{
		Process::Start("http://www.voip-sip-sdk.com/");
	}
	catch (Exception ^exception)
	{
		MessageBox::Show(exception->Message);
	}
}

void WindowMain::Tsmi_About_Click(System::Object ^sender, System::EventArgs ^e)
{
	auto aboutBox = gcnew AboutBox();
	aboutBox->ShowDialog(this);
}


